import datetime
from typing import Dict

import jwt
from jwt.exceptions import InvalidTokenError
from rich import print as rich_print

from sifflet.configure.service import SiffletConfig
from sifflet.errors import exception_handler
from sifflet.logger import logger
from sifflet.status.api import ApiStatus


class StatusService:
    def __init__(self, sifflet_config):
        self.sifflet_config: SiffletConfig = sifflet_config
        self.api_rules = ApiStatus(sifflet_config)

    @exception_handler
    def check_status(self):
        tenant_validity = self._check_tenant()

        token_validity = self._check_token() if tenant_validity else False

        if token_validity and tenant_validity:
            rich_print("[bold green]Status = OK[/bold green]")
        else:
            rich_print("[bold red]Status = KO[/bold red]")

    @exception_handler
    def _check_token(self):
        token_validity = False
        try:
            decoded_token: Dict = jwt.decode(self.sifflet_config.token, options={"verify_signature": False})
            expiration_time: datetime = datetime.datetime.fromtimestamp(decoded_token.get("exp"))
            rich_print(f"Token expiration date = {expiration_time}")
            token_validity = self.api_rules.fetch_token_valid()
        except InvalidTokenError as err:
            rich_print(f"[red]Error decoding token:[/] {err}")

        if token_validity:
            rich_print("[green]Token is valid with scope API[/green]")
        else:
            rich_print("[red]Token is not valid[/red]")

        return token_validity

    @exception_handler
    def _check_tenant(self):
        logger.debug(f"Tenant is set to: {self.sifflet_config.tenant}")
        tenant_validity = self.api_rules.fetch_health_tenant()

        if tenant_validity:
            logger.debug(f"Connected to tenant: {self.sifflet_config.tenant}")
            rich_print("[green]Tenant is up and reachable[/green]")
        else:
            logger.debug(f"Can't connect to tenant named: {self.sifflet_config.tenant}")
            rich_print(
                "[red]Tenant is not reachable. "
                "Please check the configuration of your tenant name and your network policy[/red]"
            )

        return tenant_validity



import client
from sifflet.configure.service import SiffletConfig
from sifflet.errors import config_needed_handler


class BaseApi:
    @config_needed_handler
    def __init__(self, sifflet_config: SiffletConfig):
        self.sifflet_config = sifflet_config
        if sifflet_config.dev_mode:
            self.host = f"http://{sifflet_config.tenant}"
        else:
            self.host = f"https://{self.sifflet_config.tenant}api.siffletdata.com"
        configuration = client.Configuration(host=self.host, access_token=self.sifflet_config.token)
        self.api = client.ApiClient(configuration)



import requests
from requests import Response, RequestException

from client.api import access_token_api
from sifflet.apis.base import BaseApi
from sifflet.errors import exception_handler
from sifflet.logger import logger


class ApiStatus(BaseApi):
    def __init__(self, sifflet_config):
        super().__init__(sifflet_config)
        self.api_instance = access_token_api.AccessTokenApi(self.api)

    def fetch_health_tenant(self) -> bool:
        logger.debug(f"Check heath tenant = {self.host}")
        path: str = f"{self.host}/actuator/health"
        try:
            response: Response = requests.get(path, timeout=60)
        except RequestException:
            return False
        return response.status_code == 200

    @exception_handler
    def fetch_token_valid(self) -> bool:
        logger.debug(f"Check token for host = {self.host}")
        return self.api_instance.access_token_validity(authorization=self.sifflet_config.token)



import json
import sys
import traceback
from functools import wraps, partial

from requests.exceptions import InvalidURL
from urllib3.exceptions import LocationParseError, HTTPError

from client import ApiException
from sifflet.configure.service import ConfigureService
from sifflet.constants import TENANT_KEY_OS, TOKEN_KEY_OS
from sifflet.logger import logger


class SiffletConfigError(Exception):
    """Raised when missing an environment variable"""


class SiffletRuntimeError(Exception):
    """Raised on error sifflet SDK"""


class SiffletRunRuleFail(Exception):
    """Raised when a rule is on fail"""


def exception_handler(func):
    if not func:
        return partial(exception_handler)

    @wraps(func)
    def inner_function(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except (InvalidURL, LocationParseError) as err:
            logger.debug(traceback.format_exc())
            logger.error(f"Unable to parse url, please check the configuration of your tenant name. Error='{err}'")
            sys.exit(1)
        except HTTPError as err:
            logger.debug(traceback.format_exc())
            logger.error(f'Unable to call Sifflet API, please check configuration. Error="{err}"')
            sys.exit(1)
        except ApiException as err:
            logger.debug(traceback.format_exc())
            if err.body:
                error_body = json.loads(err.body)
                logger.error(
                    f"{err.status} - {error_body['title'] if 'title' in error_body else ''} -"
                    f" {error_body['detail'] if 'detail' in error_body else ''}"
                )
            else:
                logger.error(err)
            sys.exit(1)
        except SiffletRuntimeError as err:
            logger.debug(traceback.format_exc())
            logger.error(err)
            sys.exit(1)
        except SiffletRunRuleFail as err:
            logger.debug(traceback.format_exc())
            logger.error(err)
            sys.exit(1)
        except Exception as err:  # pylint: disable=W0703
            logger.debug(traceback.format_exc())
            logger.error(err)
            sys.exit(1)

    return inner_function


def config_needed_handler(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        sifflet_config_ = args[1]
        sifflet_config = ConfigureService().load_configuration(
            dev_mode=sifflet_config_.dev_mode, debug=sifflet_config_.debug, previous_config=sifflet_config_
        )

        if sifflet_config is None or not sifflet_config.token and not sifflet_config.tenant:
            logger.error(
                f"The environment variables [italic]{TENANT_KEY_OS}[/] and [italic]{TOKEN_KEY_OS}[/] must be set. "
                "You can also configure credentials using [bold]sifflet configure[/bold]"
            )
            sys.exit(1)
        elif not sifflet_config.token:
            logger.error(
                f"The environment variable [italic]{TOKEN_KEY_OS}[/] must be set. "
                "You can also configure it using [bold]sifflet configure[/bold]"
            )
            sys.exit(1)
        elif not sifflet_config.tenant:
            logger.error(
                f"The environment variable [italic]{TENANT_KEY_OS}[/] must be set. "
                "You can also configure it using [bold]sifflet configure[/bold]"
            )
            sys.exit(1)
        return func(*args, **kwargs)

    return wrapper



"""
    Sifflet Backend API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from client.api_client import ApiClient, Endpoint as _Endpoint
from client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from client.model.get_sifflet_rule_graph_by_run_and_group200_response import GetSiffletRuleGraphByRunAndGroup200Response
from client.model.monitoring_search_dto import MonitoringSearchDto
from client.model.problem import Problem
from client.model.rule_details_dto import RuleDetailsDto
from client.model.rule_dry_run_dto import RuleDryRunDto
from client.model.rule_dry_run_result_dto import RuleDryRunResultDto
from client.model.rule_dto import RuleDto
from client.model.rule_info_dto import RuleInfoDto
from client.model.rule_overview_dto import RuleOverviewDto
from client.model.rule_run_dto import RuleRunDto


class RuleApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.create_sifflet_rule_endpoint = _Endpoint(
            settings={
                'response_type': (RuleDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules',
                'operation_id': 'create_sifflet_rule',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'rule_dto',
                ],
                'required': [
                    'rule_dto',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'rule_dto':
                        (RuleDto,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'rule_dto': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.delete_sifflet_rule_by_id_endpoint = _Endpoint(
            settings={
                'response_type': (RuleDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}',
                'operation_id': 'delete_sifflet_rule_by_id',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_all_rule_endpoint = _Endpoint(
            settings={
                'response_type': (MonitoringSearchDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules',
                'operation_id': 'get_all_rule',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'text_search',
                    'last_run_status',
                    'tag',
                    'dataset',
                    'datasource',
                    'rule_template_name',
                    'page',
                    'items_per_page',
                    'sort',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'text_search':
                        (str,),
                    'last_run_status':
                        ([str],),
                    'tag':
                        ([str],),
                    'dataset':
                        ([str],),
                    'datasource':
                        ([str],),
                    'rule_template_name':
                        ([str],),
                    'page':
                        (int,),
                    'items_per_page':
                        (int,),
                    'sort':
                        ([str],),
                },
                'attribute_map': {
                    'text_search': 'textSearch',
                    'last_run_status': 'lastRunStatus',
                    'tag': 'tag',
                    'dataset': 'dataset',
                    'datasource': 'datasource',
                    'rule_template_name': 'ruleTemplateName',
                    'page': 'page',
                    'items_per_page': 'itemsPerPage',
                    'sort': 'sort',
                },
                'location_map': {
                    'text_search': 'query',
                    'last_run_status': 'query',
                    'tag': 'query',
                    'dataset': 'query',
                    'datasource': 'query',
                    'rule_template_name': 'query',
                    'page': 'query',
                    'items_per_page': 'query',
                    'sort': 'query',
                },
                'collection_format_map': {
                    'last_run_status': 'multi',
                    'tag': 'multi',
                    'dataset': 'multi',
                    'datasource': 'multi',
                    'rule_template_name': 'multi',
                    'sort': 'multi',
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_sifflet_rule_by_id_endpoint = _Endpoint(
            settings={
                'response_type': (RuleDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}',
                'operation_id': 'get_sifflet_rule_by_id',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_sifflet_rule_details_endpoint = _Endpoint(
            settings={
                'response_type': (RuleDetailsDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}/details',
                'operation_id': 'get_sifflet_rule_details',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_sifflet_rule_graph_endpoint = _Endpoint(
            settings={
                'response_type': (GetSiffletRuleGraphByRunAndGroup200Response,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}/graph',
                'operation_id': 'get_sifflet_rule_graph',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_sifflet_rule_graph_by_run_and_group_endpoint = _Endpoint(
            settings={
                'response_type': (GetSiffletRuleGraphByRunAndGroup200Response,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}/runs/{runId}/graph/{groupValue}',
                'operation_id': 'get_sifflet_rule_graph_by_run_and_group',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'run_id',
                    'group_value',
                ],
                'required': [
                    'id',
                    'run_id',
                    'group_value',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'run_id':
                        (str,),
                    'group_value':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'run_id': 'runId',
                    'group_value': 'groupValue',
                },
                'location_map': {
                    'id': 'path',
                    'run_id': 'path',
                    'group_value': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_sifflet_rule_info_endpoint = _Endpoint(
            settings={
                'response_type': (RuleInfoDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}/info',
                'operation_id': 'get_sifflet_rule_info',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_sifflet_rule_overview_endpoint = _Endpoint(
            settings={
                'response_type': (RuleOverviewDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}/overview',
                'operation_id': 'get_sifflet_rule_overview',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.patch_rule_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}',
                'operation_id': 'patch_rule',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'body',
                ],
                'required': [
                    'id',
                    'body',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'body':
                        ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                    'body': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.sifflet_rule_dry_run_endpoint = _Endpoint(
            settings={
                'response_type': (RuleDryRunResultDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/_dry-run',
                'operation_id': 'sifflet_rule_dry_run',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'rule_dry_run_dto',
                ],
                'required': [
                    'rule_dry_run_dto',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'rule_dry_run_dto':
                        (RuleDryRunDto,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'rule_dry_run_dto': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.sifflet_rule_manual_run_endpoint = _Endpoint(
            settings={
                'response_type': (RuleRunDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}/_run',
                'operation_id': 'sifflet_rule_manual_run',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                ],
                'required': [
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.update_sifflet_rule_endpoint = _Endpoint(
            settings={
                'response_type': (RuleDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/rules/{id}',
                'operation_id': 'update_sifflet_rule',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'rule_dto',
                ],
                'required': [
                    'id',
                    'rule_dto',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'rule_dto':
                        (RuleDto,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                    'rule_dto': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )

    def create_sifflet_rule(
        self,
        rule_dto,
        **kwargs
    ):
        """Create a Sifflet rule  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_sifflet_rule(rule_dto, async_req=True)
        >>> result = thread.get()

        Args:
            rule_dto (RuleDto):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['rule_dto'] = \
            rule_dto
        return self.create_sifflet_rule_endpoint.call_with_http_info(**kwargs)

    def delete_sifflet_rule_by_id(
        self,
        id,
        **kwargs
    ):
        """Delete Sifflet rule by given id  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_sifflet_rule_by_id(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.delete_sifflet_rule_by_id_endpoint.call_with_http_info(**kwargs)

    def get_all_rule(
        self,
        **kwargs
    ):
        """Fetch rule catalog assets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_all_rule(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            text_search (str): Global text search. [optional]
            last_run_status ([str]): Filter on given last run statuses. [optional]
            tag ([str]): Filter on given tag ids. [optional]
            dataset ([str]): Filter on given dataset ids. [optional]
            datasource ([str]): Filter on given datasource ids. [optional]
            rule_template_name ([str]): Filter on given rule template names. [optional]
            page (int): The requested page number. Zero-based page index (0..N). [optional] if omitted the server will use the default value of 0
            items_per_page (int): The number of elements to be returned inside the page. Pass a value of -1, to bypass pagination and fetch all items. [optional] if omitted the server will use the default value of 25
            sort ([str]): The resource fields on which to apply the sort, format : property,ASC|DESC. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            MonitoringSearchDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.get_all_rule_endpoint.call_with_http_info(**kwargs)

    def get_sifflet_rule_by_id(
        self,
        id,
        **kwargs
    ):
        """Get a single Sifflet rule  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_sifflet_rule_by_id(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.get_sifflet_rule_by_id_endpoint.call_with_http_info(**kwargs)

    def get_sifflet_rule_details(
        self,
        id,
        **kwargs
    ):
        """Get a single Sifflet rule details  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_sifflet_rule_details(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleDetailsDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.get_sifflet_rule_details_endpoint.call_with_http_info(**kwargs)

    def get_sifflet_rule_graph(
        self,
        id,
        **kwargs
    ):
        """Get a single Sifflet rule graph  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_sifflet_rule_graph(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GetSiffletRuleGraphByRunAndGroup200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.get_sifflet_rule_graph_endpoint.call_with_http_info(**kwargs)

    def get_sifflet_rule_graph_by_run_and_group(
        self,
        id,
        run_id,
        group_value,
        **kwargs
    ):
        """Get a single Sifflet rule graph  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_sifflet_rule_graph_by_run_and_group(id, run_id, group_value, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):
            run_id (str):
            group_value (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GetSiffletRuleGraphByRunAndGroup200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['run_id'] = \
            run_id
        kwargs['group_value'] = \
            group_value
        return self.get_sifflet_rule_graph_by_run_and_group_endpoint.call_with_http_info(**kwargs)

    def get_sifflet_rule_info(
        self,
        id,
        **kwargs
    ):
        """Get a single Sifflet rule info  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_sifflet_rule_info(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleInfoDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.get_sifflet_rule_info_endpoint.call_with_http_info(**kwargs)

    def get_sifflet_rule_overview(
        self,
        id,
        **kwargs
    ):
        """Get a single Sifflet rule overview  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_sifflet_rule_overview(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleOverviewDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.get_sifflet_rule_overview_endpoint.call_with_http_info(**kwargs)

    def patch_rule(
        self,
        id,
        body,
        **kwargs
    ):
        """Patch Rule  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.patch_rule(id, body, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):
            body ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['body'] = \
            body
        return self.patch_rule_endpoint.call_with_http_info(**kwargs)

    def sifflet_rule_dry_run(
        self,
        rule_dry_run_dto,
        **kwargs
    ):
        """Trigger a dry run of Sifflet Rule   # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.sifflet_rule_dry_run(rule_dry_run_dto, async_req=True)
        >>> result = thread.get()

        Args:
            rule_dry_run_dto (RuleDryRunDto):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleDryRunResultDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['rule_dry_run_dto'] = \
            rule_dry_run_dto
        return self.sifflet_rule_dry_run_endpoint.call_with_http_info(**kwargs)

    def sifflet_rule_manual_run(
        self,
        id,
        **kwargs
    ):
        """Trigger a manual run of given Sifflet Rule  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.sifflet_rule_manual_run(id, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleRunDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        return self.sifflet_rule_manual_run_endpoint.call_with_http_info(**kwargs)

    def update_sifflet_rule(
        self,
        id,
        rule_dto,
        **kwargs
    ):
        """Update a Sifflet rule  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.update_sifflet_rule(id, rule_dto, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):
            rule_dto (RuleDto):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            RuleDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['rule_dto'] = \
            rule_dto
        return self.update_sifflet_rule_endpoint.call_with_http_info(**kwargs)




"""
    Sifflet Backend API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from client.api_client import ApiClient, Endpoint as _Endpoint
from client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from client.model.asset_dto import AssetDto
from client.model.assets_catalog_dto import AssetsCatalogDto
from client.model.field_dto import FieldDto
from client.model.get_asset_overview_by_urn200_response import GetAssetOverviewByUrn200Response
from client.model.problem import Problem
from client.model.schema_version_dto import SchemaVersionDto


class AssetApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.get_asset_by_urn_endpoint = _Endpoint(
            settings={
                'response_type': (AssetDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}',
                'operation_id': 'get_asset_by_urn',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                ],
                'required': [
                    'urn',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'urn':
                        (str,),
                },
                'attribute_map': {
                    'urn': 'urn',
                },
                'location_map': {
                    'urn': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_asset_fields_with_type_option_endpoint = _Endpoint(
            settings={
                'response_type': ([FieldDto],),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}/_get-fields',
                'operation_id': 'get_asset_fields_with_type_option',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                    'type',
                ],
                'required': [
                    'urn',
                    'type',
                ],
                'nullable': [
                ],
                'enum': [
                    'type',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('type',): {

                        "ANY": "any",
                        "DATE": "date",
                        "TIME": "time",
                        "TEMPORAL": "temporal"
                    },
                },
                'openapi_types': {
                    'urn':
                        (str,),
                    'type':
                        (str,),
                },
                'attribute_map': {
                    'urn': 'urn',
                    'type': 'type',
                },
                'location_map': {
                    'urn': 'path',
                    'type': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_asset_overview_by_urn_endpoint = _Endpoint(
            settings={
                'response_type': (GetAssetOverviewByUrn200Response,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}/overview',
                'operation_id': 'get_asset_overview_by_urn',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                ],
                'required': [
                    'urn',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'urn':
                        (str,),
                },
                'attribute_map': {
                    'urn': 'urn',
                },
                'location_map': {
                    'urn': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_asset_preview_by_urn_endpoint = _Endpoint(
            settings={
                'response_type': ([{str: ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},)}],),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}/preview',
                'operation_id': 'get_asset_preview_by_urn',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                    'limit',
                    'fields',
                ],
                'required': [
                    'urn',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'urn':
                        (str,),
                    'limit':
                        (int,),
                    'fields':
                        ([str],),
                },
                'attribute_map': {
                    'urn': 'urn',
                    'limit': 'limit',
                    'fields': 'fields',
                },
                'location_map': {
                    'urn': 'path',
                    'limit': 'query',
                    'fields': 'query',
                },
                'collection_format_map': {
                    'fields': 'multi',
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_asset_schema_by_urn_and_version_endpoint = _Endpoint(
            settings={
                'response_type': ([FieldDto],),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}/schema/{version}',
                'operation_id': 'get_asset_schema_by_urn_and_version',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                    'version',
                ],
                'required': [
                    'urn',
                    'version',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'urn':
                        (str,),
                    'version':
                        (str,),
                },
                'attribute_map': {
                    'urn': 'urn',
                    'version': 'version',
                },
                'location_map': {
                    'urn': 'path',
                    'version': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_asset_schema_versions_by_urn_endpoint = _Endpoint(
            settings={
                'response_type': ([SchemaVersionDto],),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}/schema-versions',
                'operation_id': 'get_asset_schema_versions_by_urn',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                ],
                'required': [
                    'urn',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'urn':
                        (str,),
                },
                'attribute_map': {
                    'urn': 'urn',
                },
                'location_map': {
                    'urn': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_search_results_and_filter_endpoint = _Endpoint(
            settings={
                'response_type': (AssetsCatalogDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/_search-assets',
                'operation_id': 'get_search_results_and_filter',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'page_num',
                    'page_size',
                    'search',
                    'sort_by',
                    'sort_desc',
                    'datasources',
                    'tags',
                    'health',
                    'asset_type',
                ],
                'required': [
                    'page_num',
                    'page_size',
                ],
                'nullable': [
                ],
                'enum': [
                    'health',
                    'asset_type',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('health',): {

                        "CRITICAL": "CRITICAL",
                        "AT_RISK": "AT_RISK",
                        "HEALTHY": "HEALTHY",
                        "UNMONITORED": "UNMONITORED",
                        "NOT_SUPPORTED": "NOT_SUPPORTED"
                    },
                    ('asset_type',): {

                        "TABLES": "TABLES",
                        "DASHBOARDS": "DASHBOARDS",
                        "METRICS": "METRICS",
                        "MODELS": "MODELS",
                        "DAGS": "DAGS"
                    },
                },
                'openapi_types': {
                    'page_num':
                        (int,),
                    'page_size':
                        (int,),
                    'search':
                        (str,),
                    'sort_by':
                        ([str],),
                    'sort_desc':
                        ([bool],),
                    'datasources':
                        ([str],),
                    'tags':
                        ([str],),
                    'health':
                        ([str],),
                    'asset_type':
                        ([str],),
                },
                'attribute_map': {
                    'page_num': 'pageNum',
                    'page_size': 'pageSize',
                    'search': 'search',
                    'sort_by': 'sortBy',
                    'sort_desc': 'sortDesc',
                    'datasources': 'datasources',
                    'tags': 'tags',
                    'health': 'health',
                    'asset_type': 'assetType',
                },
                'location_map': {
                    'page_num': 'query',
                    'page_size': 'query',
                    'search': 'query',
                    'sort_by': 'query',
                    'sort_desc': 'query',
                    'datasources': 'query',
                    'tags': 'query',
                    'health': 'query',
                    'asset_type': 'query',
                },
                'collection_format_map': {
                    'sort_by': 'multi',
                    'sort_desc': 'multi',
                    'datasources': 'multi',
                    'tags': 'multi',
                    'health': 'multi',
                    'asset_type': 'multi',
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.patch_asset_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/assets/{urn}',
                'operation_id': 'patch_asset',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'urn',
                    'body',
                ],
                'required': [
                    'urn',
                    'body',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'urn':
                        (str,),
                    'body':
                        ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),
                },
                'attribute_map': {
                    'urn': 'urn',
                },
                'location_map': {
                    'urn': 'path',
                    'body': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )

    def get_asset_by_urn(
        self,
        urn,
        **kwargs
    ):
        """Get a single asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_asset_by_urn(urn, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            AssetDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        return self.get_asset_by_urn_endpoint.call_with_http_info(**kwargs)

    def get_asset_fields_with_type_option(
        self,
        urn,
        type,
        **kwargs
    ):
        """Get fields with type option  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_asset_fields_with_type_option(urn, type, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):
            type (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [FieldDto]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        kwargs['type'] = \
            type
        return self.get_asset_fields_with_type_option_endpoint.call_with_http_info(**kwargs)

    def get_asset_overview_by_urn(
        self,
        urn,
        **kwargs
    ):
        """Get a single asset overview params  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_asset_overview_by_urn(urn, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GetAssetOverviewByUrn200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        return self.get_asset_overview_by_urn_endpoint.call_with_http_info(**kwargs)

    def get_asset_preview_by_urn(
        self,
        urn,
        **kwargs
    ):
        """Get a single asset preview data  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_asset_preview_by_urn(urn, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):

        Keyword Args:
            limit (int): Number of rows. [optional] if omitted the server will use the default value of 25
            fields ([str]): Columns requested. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [{str: ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},)}]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        return self.get_asset_preview_by_urn_endpoint.call_with_http_info(**kwargs)

    def get_asset_schema_by_urn_and_version(
        self,
        urn,
        version,
        **kwargs
    ):
        """Get a single asset schema  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_asset_schema_by_urn_and_version(urn, version, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):
            version (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [FieldDto]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        kwargs['version'] = \
            version
        return self.get_asset_schema_by_urn_and_version_endpoint.call_with_http_info(**kwargs)

    def get_asset_schema_versions_by_urn(
        self,
        urn,
        **kwargs
    ):
        """Get a single asset schema versions  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_asset_schema_versions_by_urn(urn, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [SchemaVersionDto]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        return self.get_asset_schema_versions_by_urn_endpoint.call_with_http_info(**kwargs)

    def get_search_results_and_filter(
        self,
        page_num,
        page_size,
        **kwargs
    ):
        """Get search results and search filters for asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_search_results_and_filter(page_num, page_size, async_req=True)
        >>> result = thread.get()

        Args:
            page_num (int):
            page_size (int):

        Keyword Args:
            search (str): [optional]
            sort_by ([str]): [optional]
            sort_desc ([bool]): [optional]
            datasources ([str]): [optional]
            tags ([str]): [optional]
            health ([str]): [optional]
            asset_type ([str]): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            AssetsCatalogDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['page_num'] = \
            page_num
        kwargs['page_size'] = \
            page_size
        return self.get_search_results_and_filter_endpoint.call_with_http_info(**kwargs)

    def patch_asset(
        self,
        urn,
        body,
        **kwargs
    ):
        """Patch Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.patch_asset(urn, body, async_req=True)
        >>> result = thread.get()

        Args:
            urn (str):
            body ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['urn'] = \
            urn
        kwargs['body'] = \
            body
        return self.patch_asset_endpoint.call_with_http_info(**kwargs)




from typing import List, Tuple

from client.api import rule_run_api, rule_api, sifflet_rule_run_api
from client.model.rule_catalog_asset_dto import RuleCatalogAssetDto
from client.model.rule_info_dto import RuleInfoDto
from client.model.rule_run_dto import RuleRunDto
from client.model.search_collection_rule_catalog_asset_dto import SearchCollectionRuleCatalogAssetDto
from sifflet.apis.base import BaseApi
from sifflet.constants import DEFAULT_PAGE_NUM, DEFAULT_PAGE_SIZE
from sifflet.errors import exception_handler
from sifflet.logger import logger


class RulesApi(BaseApi):
    def __init__(self, sifflet_config):
        super().__init__(sifflet_config)
        self.api_instance_rule = rule_api.RuleApi(self.api)
        self.api_instance_rule_run = rule_run_api.RuleRunApi(self.api)
        self.api_instance_sifflet_rule_run = sifflet_rule_run_api.SiffletRuleRunApi(self.api)

    def fetch_rules(
        self, filter_: str, page_size=DEFAULT_PAGE_SIZE, page_num=DEFAULT_PAGE_NUM
    ) -> Tuple[List[RuleCatalogAssetDto], int]:
        """Get a list of rules for a given filter"""
        logger.debug(f"Fetch rules with search filter = {filter_}, page_size={page_size}, page_num={page_num}")
        if not filter_:
            filter_ = ""
        response: SearchCollectionRuleCatalogAssetDto = self.api_instance_rule.get_all_rule(
            items_per_page=page_size,
            page=page_num,
            text_search=filter_,
            sort=[RuleCatalogAssetDto.attribute_map["name"] + ",ASC"],
        )
        return response.get("search_rules").get("data"), response.get("search_rules").get("total_elements")

    @exception_handler
    def run_rule(self, id_: str) -> RuleRunDto:
        """Run a rule and return the runRunId of the created run"""
        logger.debug(f"Run rule_id = {id_}")
        # TODO: backend should check if the rule is already running, and throw an exception if running
        return self.api_instance_rule.sifflet_rule_manual_run(id=id_)

    def status_rule_run(self, run_id: str, rule_id: str) -> RuleRunDto:
        """Get details of a RuleRun"""
        logger.debug(f"Fetch rule run, id = {run_id} of rule {rule_id}")
        return self.api_instance_sifflet_rule_run.get_sifflet_rule_run(id=rule_id, run_id=run_id)

    def info_rule(self, rule_id: str) -> RuleInfoDto:
        """Get details of a Rule"""
        logger.debug(f"Fetch rule, id = {rule_id}")
        return self.api_instance_rule.get_sifflet_rule_info(id=rule_id)

    def rule_runs(self, id_: str, page_size=DEFAULT_PAGE_SIZE, page=DEFAULT_PAGE_NUM) -> Tuple[List[RuleRunDto], int]:
        """Get a list of RuleRun for a given ruleId"""
        logger.debug(f"Fetch rule run, rule_id = {id_}")
        response = self.api_instance_rule_run.get_sifflet_rule_runs(
            id=id_,
            items_per_page=page_size,
            page=page,
            sort=[RuleRunDto.attribute_map["start_date"] + ",DESC"],
        )
        return response.get("data"), response.get("total_elements")



from sifflet.configure.service import SiffletConfig
from sifflet.errors import exception_handler
from sifflet.ingest.api import ApiIngestion


class IngestionService:
    def __init__(self, sifflet_config):
        self.sifflet_config: SiffletConfig = sifflet_config
        self.api_ingestion = ApiIngestion(sifflet_config)

    @exception_handler
    def ingest_dbt(self, project_name: str, target: str, input_folder: str) -> bool:
        return self.api_ingestion.send_dbt_metadata(project_name, target, input_folder)



"""
    Sifflet Backend API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from client.api_client import ApiClient, Endpoint as _Endpoint
from client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from client.model.create_comment_dto import CreateCommentDto
from client.model.event_dto import EventDto
from client.model.incident_detail_dto import IncidentDetailDto
from client.model.issue_details_dto import IssueDetailsDto
from client.model.issue_overview_dto import IssueOverviewDto
from client.model.lineage_entity_dto import LineageEntityDto
from client.model.problem import Problem
from client.model.resource_ids_dto import ResourceIdsDto
from client.model.search_collection_incident_light_dto import SearchCollectionIncidentLightDto


class IncidentApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.add_comment_endpoint = _Endpoint(
            settings={
                'response_type': (EventDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{id}/_comment',
                'operation_id': 'add_comment',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'create_comment_dto',
                ],
                'required': [
                    'id',
                    'create_comment_dto',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'create_comment_dto':
                        (CreateCommentDto,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                    'create_comment_dto': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.assign_endpoint = _Endpoint(
            settings={
                'response_type': (IncidentDetailDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{id}/_assign',
                'operation_id': 'assign',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'resource_ids_dto',
                ],
                'required': [
                    'id',
                    'resource_ids_dto',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'resource_ids_dto':
                        (ResourceIdsDto,),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                    'resource_ids_dto': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )
        self.get_all_incident_endpoint = _Endpoint(
            settings={
                'response_type': (SearchCollectionIncidentLightDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents',
                'operation_id': 'get_all_incident',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'text_search',
                    '_from',
                    'to',
                    'criticality',
                    'status',
                    'user',
                    'rule',
                    'dataset',
                    'page',
                    'items_per_page',
                    'sort',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'text_search':
                        (str,),
                    '_from':
                        (int,),
                    'to':
                        (int,),
                    'criticality':
                        ([int],),
                    'status':
                        ([str],),
                    'user':
                        ([str],),
                    'rule':
                        ([str],),
                    'dataset':
                        ([str],),
                    'page':
                        (int,),
                    'items_per_page':
                        (int,),
                    'sort':
                        ([str],),
                },
                'attribute_map': {
                    'text_search': 'textSearch',
                    '_from': 'from',
                    'to': 'to',
                    'criticality': 'criticality',
                    'status': 'status',
                    'user': 'user',
                    'rule': 'rule',
                    'dataset': 'dataset',
                    'page': 'page',
                    'items_per_page': 'itemsPerPage',
                    'sort': 'sort',
                },
                'location_map': {
                    'text_search': 'query',
                    '_from': 'query',
                    'to': 'query',
                    'criticality': 'query',
                    'status': 'query',
                    'user': 'query',
                    'rule': 'query',
                    'dataset': 'query',
                    'page': 'query',
                    'items_per_page': 'query',
                    'sort': 'query',
                },
                'collection_format_map': {
                    'criticality': 'multi',
                    'status': 'multi',
                    'user': 'multi',
                    'rule': 'multi',
                    'dataset': 'multi',
                    'sort': 'multi',
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_incident_by_issue_number_endpoint = _Endpoint(
            settings={
                'response_type': (IssueDetailsDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{issueNo}',
                'operation_id': 'get_incident_by_issue_number',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'issue_no',
                ],
                'required': [
                    'issue_no',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'issue_no':
                        (int,),
                },
                'attribute_map': {
                    'issue_no': 'issueNo',
                },
                'location_map': {
                    'issue_no': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_incident_downstream_impact_by_issue_number_endpoint = _Endpoint(
            settings={
                'response_type': ([LineageEntityDto],),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{issueNo}/downstream-impact',
                'operation_id': 'get_incident_downstream_impact_by_issue_number',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'issue_no',
                ],
                'required': [
                    'issue_no',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'issue_no':
                        (int,),
                },
                'attribute_map': {
                    'issue_no': 'issueNo',
                },
                'location_map': {
                    'issue_no': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_incident_events_by_issue_number_endpoint = _Endpoint(
            settings={
                'response_type': ([EventDto],),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{issueNo}/events',
                'operation_id': 'get_incident_events_by_issue_number',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'issue_no',
                ],
                'required': [
                    'issue_no',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'issue_no':
                        (int,),
                },
                'attribute_map': {
                    'issue_no': 'issueNo',
                },
                'location_map': {
                    'issue_no': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/problem+json',
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.get_issue_overview_by_issue_number_endpoint = _Endpoint(
            settings={
                'response_type': (IssueOverviewDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{issueNo}/overview',
                'operation_id': 'get_issue_overview_by_issue_number',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'issue_no',
                ],
                'required': [
                    'issue_no',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'issue_no':
                        (int,),
                },
                'attribute_map': {
                    'issue_no': 'issueNo',
                },
                'location_map': {
                    'issue_no': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.patch_incident_endpoint = _Endpoint(
            settings={
                'response_type': (IncidentDetailDto,),
                'auth': [
                    'JWT'
                ],
                'endpoint_path': '/api/v1/incidents/{id}',
                'operation_id': 'patch_incident',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'body',
                ],
                'required': [
                    'id',
                    'body',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'body':
                        ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),
                },
                'attribute_map': {
                    'id': 'id',
                },
                'location_map': {
                    'id': 'path',
                    'body': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json',
                    'application/problem+json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )

    def add_comment(
        self,
        id,
        create_comment_dto,
        **kwargs
    ):
        """Comment on an incident  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.add_comment(id, create_comment_dto, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):
            create_comment_dto (CreateCommentDto):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            EventDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['create_comment_dto'] = \
            create_comment_dto
        return self.add_comment_endpoint.call_with_http_info(**kwargs)

    def assign(
        self,
        id,
        resource_ids_dto,
        **kwargs
    ):
        """Assign one or more users to an incident  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.assign(id, resource_ids_dto, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):
            resource_ids_dto (ResourceIdsDto):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            IncidentDetailDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['resource_ids_dto'] = \
            resource_ids_dto
        return self.assign_endpoint.call_with_http_info(**kwargs)

    def get_all_incident(
        self,
        **kwargs
    ):
        """Get incidents  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_all_incident(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            text_search (str): Global text search. [optional]
            _from (int): Retrieve incidents from the given start date. [optional]
            to (int): Retrieve incidents up to a given end date. [optional]
            criticality ([int]): [optional]
            status ([str]): Filter on given statuses, allowable values are UNASSIGNED, ONGOING and CLOSED. [optional]
            user ([str]): Filter on given user ids. [optional]
            rule ([str]): Filter on given rule type. [optional]
            dataset ([str]): Filter on given dataset ids. [optional]
            page (int): The requested page number. Zero-based page index (0..N). [optional] if omitted the server will use the default value of 0
            items_per_page (int): The number of elements to be returned inside the page. Pass a value of -1, to bypass pagination and fetch all items. [optional] if omitted the server will use the default value of 25
            sort ([str]): The resource fields on which to apply the sort, format : property,ASC|DESC. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            SearchCollectionIncidentLightDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.get_all_incident_endpoint.call_with_http_info(**kwargs)

    def get_incident_by_issue_number(
        self,
        issue_no,
        **kwargs
    ):
        """Get single incident by issue number  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_incident_by_issue_number(issue_no, async_req=True)
        >>> result = thread.get()

        Args:
            issue_no (int):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            IssueDetailsDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['issue_no'] = \
            issue_no
        return self.get_incident_by_issue_number_endpoint.call_with_http_info(**kwargs)

    def get_incident_downstream_impact_by_issue_number(
        self,
        issue_no,
        **kwargs
    ):
        """Get incident impact on lineage downstream  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_incident_downstream_impact_by_issue_number(issue_no, async_req=True)
        >>> result = thread.get()

        Args:
            issue_no (int):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [LineageEntityDto]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['issue_no'] = \
            issue_no
        return self.get_incident_downstream_impact_by_issue_number_endpoint.call_with_http_info(**kwargs)

    def get_incident_events_by_issue_number(
        self,
        issue_no,
        **kwargs
    ):
        """Get an incident events  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_incident_events_by_issue_number(issue_no, async_req=True)
        >>> result = thread.get()

        Args:
            issue_no (int):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            [EventDto]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['issue_no'] = \
            issue_no
        return self.get_incident_events_by_issue_number_endpoint.call_with_http_info(**kwargs)

    def get_issue_overview_by_issue_number(
        self,
        issue_no,
        **kwargs
    ):
        """Get single incident overview by issue number  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_issue_overview_by_issue_number(issue_no, async_req=True)
        >>> result = thread.get()

        Args:
            issue_no (int):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            IssueOverviewDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['issue_no'] = \
            issue_no
        return self.get_issue_overview_by_issue_number_endpoint.call_with_http_info(**kwargs)

    def patch_incident(
        self,
        id,
        body,
        **kwargs
    ):
        """Patch an incident  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.patch_incident(id, body, async_req=True)
        >>> result = thread.get()

        Args:
            id (str):
            body ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            IncidentDetailDto
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['body'] = \
            body
        return self.patch_incident_endpoint.call_with_http_info(**kwargs)




import configparser
import logging
import os
from dataclasses import dataclass
from pathlib import Path

from sifflet.constants import (
    TENANT_KEY,
    TOKEN_KEY,
    DEV_MODE_KEY,
    DEBUG_KEY,
    APP_SECTION_KEY,
    TENANT_KEY_OS,
    TOKEN_KEY_OS,
)
from sifflet.logger import logger


@dataclass
class SiffletConfig:
    tenant: str
    token: str
    debug: bool = False
    dev_mode: bool = False


class ConfigureService:
    path_folder_config = Path.home() / ".sifflet"
    path_file_config = path_folder_config / "config.ini"

    def configure(self, tenant, token) -> SiffletConfig:
        sifflet_config = SiffletConfig(
            tenant=tenant,
            token=token,
        )

        config = configparser.ConfigParser()
        config[APP_SECTION_KEY] = {
            TENANT_KEY: sifflet_config.tenant,
            TOKEN_KEY: sifflet_config.token,
            DEV_MODE_KEY: str(sifflet_config.dev_mode) if sifflet_config.dev_mode else "False",
            DEBUG_KEY: str(sifflet_config.debug) or "False",
        }

        self.path_folder_config.mkdir(exist_ok=True, parents=True)

        with open(self.path_file_config, mode="w", encoding="utf-8") as configfile:
            config.write(configfile)

        logger.info(f"Sifflet configuration saved to {self.path_file_config}")

        return sifflet_config

    def load_configuration(self, debug=False, dev_mode=False, previous_config: SiffletConfig = None) -> SiffletConfig:
        """
        Initialize Sifflet configuration
        First from environment variables then falls back to configuration file
        """
        if debug:
            logger.setLevel(logging.DEBUG)
            logger.debug("Log level set to DEBUG")

        config = configparser.ConfigParser()
        config.read(self.path_file_config)

        # Get env var or falls back to config file
        if "APP" in config.sections():
            default_tenant = config[APP_SECTION_KEY][TENANT_KEY]
            default_token = config[APP_SECTION_KEY][TOKEN_KEY]
        elif previous_config:
            default_tenant = previous_config.tenant or ""
            default_token = previous_config.token or ""
        else:
            default_tenant = ""
            default_token = ""
        return SiffletConfig(
            tenant=os.getenv(TENANT_KEY_OS, default=default_tenant),
            token=os.getenv(TOKEN_KEY_OS, default=default_token),
            dev_mode=dev_mode,
            debug=debug,
        )



from typing import List

import click
from click_aliases import ClickAliasedGroup

from sifflet.constants import SIFFLET_CONFIG_CTX, OutputType, DEFAULT_PAGE_SIZE, DEFAULT_PAGE_NUM
from sifflet.rules.service import RulesService


@click.group(cls=ClickAliasedGroup)
def rules():
    """List and control rules"""


@rules.command(name="list", aliases=["ls"])
@click.option("--name", "-n", type=str, required=False, help="Search rules by name")
@click.option(
    "--output",
    "-o",
    type=click.Choice(OutputType.list(), case_sensitive=False),
    required=False,
    help="Display the result either as a Table or raw Json",
    default="table",
    show_default=True,
)
@click.option(
    "--page-size",
    "page_size",
    type=int,
    required=False,
    help="Page size of the server side pagination",
    default=DEFAULT_PAGE_SIZE,
    show_default=True,
)
@click.option(
    "--page-num",
    "page_num",
    type=int,
    required=False,
    help="Page number of the server side pagination",
    default=DEFAULT_PAGE_NUM,
    show_default=True,
)
@click.pass_context
def list_rules(
    ctx, name: str, output: str = "table", page_size: int = DEFAULT_PAGE_SIZE, page_num: int = DEFAULT_PAGE_NUM
):
    """Display all rules created"""
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    service = RulesService(sifflet_config, output_type=output, page_size=page_size, page_num=page_num)
    service.show_rules(name)


@rules.command()
@click.option("--id", "ids", multiple=True, required=True, help="The rule id to trigger")
@click.pass_context
def run(ctx, ids: List[str]):
    """Run one or several rules - requires rule id(s)"""
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    service = RulesService(sifflet_config)
    rule_runs = service.run_rules(ids)
    service.wait_rule_runs(rule_runs)


@rules.command("run-history")
@click.option("--id", "rule_id", required=True, help="id of the rules id to fetch")
@click.option(
    "--output",
    "-o",
    type=click.Choice(OutputType.list(), case_sensitive=False),
    default="table",
    required=False,
    help="Display the result either as a Table or raw Json",
)
@click.option(
    "--page-size",
    "page_size",
    type=int,
    required=False,
    help="Page size of the server side pagination",
    default=DEFAULT_PAGE_SIZE,
    show_default=True,
)
@click.option(
    "--page-num",
    "page_num",
    type=int,
    required=False,
    help="Page number of the server side pagination",
    default=DEFAULT_PAGE_NUM,
    show_default=True,
)
@click.pass_context
def run_history(
    ctx, rule_id: str, output: str = "table", page_size: int = DEFAULT_PAGE_SIZE, page_num: int = DEFAULT_PAGE_NUM
):
    """Display all rule runs for a given rule id"""
    sifflet_config = ctx.obj[SIFFLET_CONFIG_CTX]
    service = RulesService(sifflet_config, output_type=output, page_size=page_size, page_num=page_num)
    service.show_run_history(rule_id=rule_id)



from typing import List

from rich.console import Console
from rich.table import Table, Column

from sifflet.logger import logger


def show_table(table: List[dict], title: str = None) -> None:
    """Utility to display rich tables"""

    if table:
        header = list(table[0].keys())

        if "id" in header:
            header = list(filter(lambda x: x != "id", header))
            table_formatted = Table(Column(header="id", no_wrap=True), *header)
        else:
            table_formatted = Table(*header)

        for val in table:
            table_formatted.add_row(*val.values())

        table_formatted.title = title

        console = Console()
        console.print(table_formatted)
    else:
        logger.warning("No data to display")



import click

from sifflet.configure.service import ConfigureService, SiffletConfig
from sifflet.constants import SIFFLET_CONFIG_CTX, TENANT_KEY, TOKEN_KEY
from sifflet.logger import logger


def default_from_context(default_key):
    class DefaultFromContext(click.Option):
        def get_default(self, ctx, call=True):
            sifflet_config: SiffletConfig = ctx.obj[SIFFLET_CONFIG_CTX]
            if default_key == TENANT_KEY:
                self.default = sifflet_config.tenant
            if default_key == TOKEN_KEY:
                self.default = sifflet_config.token
            return super().get_default(ctx)

        def __str__(self):
            print(default_key)
            return "*" * 8 if default_key == TOKEN_KEY else self.default

    return DefaultFromContext


@click.command()
@click.option(
    "--tenant",
    prompt="Your tenant name",
    help="Correspond to the prefix of your sifflet url https://TENANT.siffletdata.com/",
    show_default=True,
    cls=default_from_context(TENANT_KEY),
)
@click.option(
    "--token",
    prompt="Your API access token",
    help="The access token must be generated in the Web UI of Sifflet Settings > Access-Tokens",
    hide_input=True,
    cls=default_from_context(TOKEN_KEY),
)
def configure(tenant: str, token: str):
    """Configure sifflet variables"""
    service = ConfigureService()
    logger.debug("start service")
    service.configure(tenant, token)



from pathlib import Path

from client.api import dbt_api
from sifflet.apis.base import BaseApi
from sifflet.errors import exception_handler
from sifflet.logger import logger


class ApiIngestion(BaseApi):
    def __init__(self, sifflet_config):
        super().__init__(sifflet_config)
        self.api_instance = dbt_api.DBTApi(self.api)

    @exception_handler
    def send_dbt_metadata(self, project_name, target, input_folder: str) -> bool:
        logger.debug(f"Sending dbt metadata to host = {self.host}")
        manifest_file_path = Path(input_folder) / "target" / "manifest.json"
        catalog_file_path = Path(input_folder) / "target" / "catalog.json"
        run_results_file_path = Path(input_folder) / "target" / "run_results.json"

        with open(manifest_file_path, "rb") as manifest, open(catalog_file_path, "rb") as catalog, open(
            run_results_file_path, "rb"
        ) as run_results:
            self.api_instance.upload_dbt_metadata_files(
                project_name=project_name,
                target=target,
                manifest=manifest,
                catalog=catalog,
                run_results=run_results,
            )
            return True



"""Sifflet constants"""

# Key used to access sifflet config within click context
from enum import Enum
from typing import List

# Click context
SIFFLET_CONFIG_CTX = "sifflet_config"

# Config
APP_SECTION_KEY: str = "APP"

TENANT_KEY = "TENANT"
TENANT_KEY_OS = "SIFFLET_TENANT"
TOKEN_KEY = "TOKEN"
TOKEN_KEY_OS = "SIFFLET_TOKEN"
DEV_MODE_KEY = "DEV_MODE"
DEBUG_KEY = "DEBUG"


# Pagination
DEFAULT_PAGE_NUM = 0
DEFAULT_PAGE_SIZE = 15

# API calls Rule run
DEFAULT_TIMEOUT_MINUTES = 5


# CLI options
class OutputType(Enum):
    TABLE = "table"
    JSON = "json"

    @classmethod
    def list(cls) -> List[str]:
        return [c.value for c in cls]


# Api enums
class StatusError(Enum):
    FAILED = "FAILED"
    TECHNICAL_ERROR = "TECHNICAL_ERROR"

    @classmethod
    def list(cls) -> List[str]:
        return [c.value for c in cls]


class StatusSuccess(Enum):
    SUCCESS = "SUCCESS"

    @classmethod
    def list(cls) -> List[str]:
        return [c.value for c in cls]


class StatusRunning(Enum):
    RUNNING = "RUNNING"
    PENDING = "PENDING"

    @classmethod
    def list(cls) -> List[str]:
        return [c.value for c in cls]



import json
import time
from datetime import datetime
from typing import List, Optional

from rich import markup
from rich.console import Console
from rich.table import Table

from client.model.rule_catalog_asset_dto import RuleCatalogAssetDto
from client.model.rule_info_dto import RuleInfoDto
from client.model.rule_run_dto import RuleRunDto
from sifflet.constants import (
    OutputType,
    DEFAULT_PAGE_SIZE,
    DEFAULT_PAGE_NUM,
    DEFAULT_TIMEOUT_MINUTES,
    StatusSuccess,
    StatusRunning,
    StatusError,
)
from sifflet.errors import exception_handler, SiffletRuntimeError, SiffletRunRuleFail
from sifflet.logger import logger
from sifflet.rules.api import RulesApi
from sifflet.utils import show_table


class RulesService:
    def __init__(
        self,
        sifflet_config,
        output_type: str = OutputType.TABLE.value,
        page_size=DEFAULT_PAGE_SIZE,
        page_num=DEFAULT_PAGE_NUM,
    ):
        self.api_rules = RulesApi(sifflet_config)
        self.output_type = OutputType(output_type)
        self.page_size = page_size
        self.page_num = page_num

        self.console = Console()

    @exception_handler
    def show_rules(self, filter_name: str):
        """Display rules in a table"""
        rules, total_count = self.api_rules.fetch_rules(filter_name, page_size=self.page_size, page_num=self.page_num)

        if rules:
            rules_cleaned = [
                {
                    "id": rule.get("id"),
                    "name": self._escape_markup(rule.get("name")),
                    "datasource_type": rule.get("datasource_type"),
                    "dataset_name": rule.get("dataset_name"),
                    "platform": rule.get("source_platform"),
                    "last_run_status": self._get_last_run_status(rule),
                    "last_run": self._get_last_run_timestamp(rule),
                }
                for rule in rules
            ]
            if self.output_type == OutputType.TABLE:
                table = Table()
                table.add_column("ID", no_wrap=True)
                table.add_column("Name", no_wrap=True)
                table.add_column("Datasource Type")
                table.add_column("Dataset")
                table.add_column("Platform")
                table.add_column("Last run status", justify="right")
                table.add_column("Last run date")
                for val in rules_cleaned:
                    table.add_row(*val.values())
                self.console.print(table)

                if len(rules) < int(total_count):
                    if self.page_num == 0:
                        self.console.print(f"Showing first {len(rules)} rules out of {total_count} rules")
                    else:
                        self.console.print(f"Showing {len(rules)} rules out of {total_count} rules")
            else:
                self.console.print_json(json.dumps(rules_cleaned))
        elif filter_name:
            logger.info(f"No rule found for search filter: [bold]{filter_name}[/]")
        else:
            logger.info("No rule found")

    @staticmethod
    def _get_last_run_timestamp(rule: RuleCatalogAssetDto) -> str:
        if rule.get("last_run_status") and rule.get("last_run_status").timestamp:
            return str(datetime.fromtimestamp(rule.get("last_run_status").timestamp / 1000))
        else:
            return ""

    def _get_last_run_status(self, rule: RuleCatalogAssetDto) -> str:
        if rule.get("last_run_status") and rule.get("last_run_status").status:
            return self._format_status(rule.get("last_run_status").status)
        else:
            return ""

    @exception_handler
    def run_rules(self, rule_ids: List[str]) -> List[RuleRunDto]:
        rule_runs: List[RuleRunDto] = []
        for rule_id in rule_ids:
            logger.info(f"Triggering rule {rule_id} ...")
            rule_run: RuleRunDto = self.api_rules.run_rule(rule_id)
            rule_runs.append(rule_run)
            logger.info(f"Rule {rule_id} triggered, waiting for result...")
        return rule_runs

    @exception_handler
    def wait_rule_runs(
        self,
        rule_runs: List[RuleRunDto],
        timeout: Optional[int] = 60 * DEFAULT_TIMEOUT_MINUTES,
        wait_time: int = 2,
        error_on_rule_fail: bool = True,
    ):
        start = time.monotonic()
        rule_run_fail: List[RuleRunDto] = []

        for rule_run in rule_runs:
            result_status = self._wait_run(rule_run=rule_run, timeout=timeout, start=start, wait_time=wait_time)
            if result_status in StatusSuccess.list():
                logger.info(f"Rule success, id = '{rule_run.rule_id}'")
            else:
                logger.error(f"Rule failed, id = '{rule_run.get('id')}', result = '{rule_run.get('result')}'")
                rule_run_fail.append(rule_run)

        if rule_run_fail:
            details_fail = [{"id": rf.rule_id, "name": self._build_rule_name(rf.rule_id)} for rf in rule_run_fail]
            if error_on_rule_fail:
                raise SiffletRunRuleFail(f"The following rules are on fail: {details_fail}")
            else:
                logger.error(f"The following rules are on fail: {details_fail}")

    def _build_rule_name(self, rule_id):
        rule_overview: RuleInfoDto = self.api_rules.info_rule(rule_id=rule_id)
        return (
            f"[{rule_overview.get('datasource_name')}][{rule_overview.get('dataset_name')}]"
            f"{rule_overview.get('name')}"
        )

    @exception_handler
    def _wait_run(self, rule_run: RuleRunDto, timeout=None, start=None, wait_time=None):
        while True:
            if timeout and start + timeout < time.monotonic():
                raise SiffletRuntimeError(f"Timeout: Sifflet rule run {rule_run} not started after {timeout}s")

            time.sleep(wait_time)
            try:
                status = self.get_status_rule_run(rule_id=rule_run.get("rule_id"), rule_run_id=rule_run.get("id"))
            except SiffletRuntimeError as err:
                logger.warning("Retrying... Sifflet API returned an error when waiting for rule run status: %s", err)
                continue

            if status in StatusRunning.list():
                continue
            if status in StatusSuccess.list() or status in StatusError.list():
                return status
            raise SiffletRuntimeError(f"Encountered unexpected status `{status}` for rule run `{rule_run}`")

    def get_status_rule_run(self, rule_id: str, rule_run_id: str) -> str:
        rule_run_dto: RuleRunDto = self.api_rules.status_rule_run(rule_id=rule_id, run_id=rule_run_id)
        logger.debug(f"Rules status = {rule_run_dto.get('status')}")
        return rule_run_dto.get("status")

    def show_run_history(self, rule_id: str):
        rule_info: RuleInfoDto = self.api_rules.info_rule(rule_id=rule_id)

        rule_runs, total_count = self.api_rules.rule_runs(rule_id, page_size=self.page_size, page=self.page_num)
        if rule_runs:
            rules_runs_cleaned = [
                {
                    "status": self._format_status(rule_run.get("status", default="")),
                    "start_date": str(datetime.fromtimestamp(rule_run.get("start_date", default=0) / 1000)),
                    "end_date": str(datetime.fromtimestamp(rule_run.get("end_date", default=0) / 1000)),
                    "type": self._escape_markup(rule_run.get("type")),
                    "result": self._escape_markup(rule_run.get("result")),
                }
                for rule_run in rule_runs
            ]
            table_title = (
                f"Rule name: [{rule_info.get('datasource_name')}]"
                f"[{rule_info.get('dataset_name')}]{rule_info.get('name')}"
            )
            if self.output_type == OutputType.TABLE:
                show_table(rules_runs_cleaned, title=table_title)
            else:
                self.console.print(rules_runs_cleaned)
            if len(rule_runs) < int(total_count):
                if self.page_num == 0:
                    self.console.print(f"Showing first {len(rule_runs)} runs out of {total_count} runs")
                else:
                    self.console.print(f"Showing {len(rule_runs)} runs out of {total_count} runs")

    def _escape_markup(self, value: str):
        return markup.escape(value) if self.output_type == OutputType.TABLE else value

    def _format_status(self, status: str) -> str:
        result = status
        if self.output_type == OutputType.TABLE:
            if status in StatusError.__members__:
                result = f"[bold red]{status}[/bold red]"
            elif status in StatusSuccess.__members__:
                result = f"[bold green]{status}[/bold green]"

        return result
